@@ -12,16 +12,15 @@ namespace kiva
 		return (f_eq(temp[0], 1.0) && f_eq(temp[1], 0.0) &&
 				f_eq(temp[2], 0.0) && f_eq(temp[3], 1.0) &&
 				f_eq(temp[4], 0.0) && f_eq(temp[5], 0.0));
-//        return (temp[0] == 1.0 && temp[1] == 0.0 && 
-//                temp[2] == 0.0 && temp[3] == 1.0 && 
+//        return (temp[0] == 1.0 && temp[1] == 0.0 &&
+//                temp[2] == 0.0 && temp[3] == 1.0 &&
 //                temp[4] == 0.0 && temp[5] == 0.0);
-               
+
     }
 
     bool only_translation(agg::trans_affine& mat, double epsilon)
     {
         double temp[6];
-        bool tmp;
         mat.store_to(temp);
 		return (f_eq(temp[0], 1.0) && f_eq(temp[1], 0.0) &&
 				f_eq(temp[2], 0.0) && f_eq(temp[3], 1.0));
@@ -40,5 +39,5 @@ namespace kiva
         m.store_to(temp);
         *tx = temp[4];
         *ty = temp[5];
-    }        
+    }
 }
