@@ -3,13 +3,12 @@ import sys
 import warnings
 
 import numpy as np
-import wx
 
 from enthought import kiva
 from enthought.kiva import affine, constants
 
 from enthought.savage.svg import svg_extras
-from enthought.savage.svg.backends.null.null_renderer import NullRenderer
+from enthought.savage.svg.backends.null.null_renderer import NullRenderer, AbstractGradientBrush
 
 # Get the Canvas class for drawing on...
 
@@ -20,7 +19,7 @@ def _GetCurrentPoint(gc):
     return gc.vertex(total_vertices-1)[0]
 
 class CompiledPath(kiva.CompiledPath):
-    
+
     AddPath = kiva.CompiledPath.add_path
     AddRectangle = kiva.CompiledPath.rect
     MoveToPoint = kiva.CompiledPath.move_to
@@ -42,7 +41,7 @@ class CompiledPath(kiva.CompiledPath):
         for i, (x1,y1, x2,y2, x3,y3, x4,y4) in enumerate(svg_extras.bezier_arc(
             x, y, x+w, y+h, theta0, dtheta)):
             self.curve_to(x2,y2, x3,y3, x4,y4)
-            
+
     def elliptical_arc_to(self, rx, ry, phi, large_arc_flag, sweep_flag, x2, y2):
         if sys.platform == 'darwin':
             x1, y1 = path.get_current_point()
@@ -52,15 +51,15 @@ class CompiledPath(kiva.CompiledPath):
                 if total_vertices == 0:
                     return (0.0, 0.0)
                 return path.vertex(total_vertices-1)[0]
-            x1, y1 = _get_current_point(self) 
-                
-        arcs = svg_extras.elliptical_arc_to(self, rx, ry, phi, 
-                                            large_arc_flag, sweep_flag, 
+            x1, y1 = _get_current_point(self)
+
+        arcs = svg_extras.elliptical_arc_to(self, rx, ry, phi,
+                                            large_arc_flag, sweep_flag,
                                             x1, y1, x2, y2)
-        
+
         for arc in arcs:
             self.curve_to(*arc)
-        
+
     def AddCircle(self, x, y, r):
         self.arc(x, y, r, 0.0, 2*pi)
 
@@ -73,7 +72,7 @@ class CompiledPath(kiva.CompiledPath):
 
 
 Canvas = kiva.Canvas
-    
+
 
 class Pen(object):
     def __init__(self, color):
@@ -84,13 +83,13 @@ class Pen(object):
         self.width = 1
         self.dasharray = None
         self.dashoffset = 0.0
-        
+
     def SetCap(self, cap):
-        self.cap = cap    
+        self.cap = cap
 
     def SetJoin(self, join):
         self.join = join
-        
+
     def SetWidth(self, width):
         self.width = width
 
@@ -111,21 +110,21 @@ class Pen(object):
         gc.set_line_width(self.width)
         if self.dasharray is not None:
             gc.set_line_dash(self.dasharray, self.dashoffset)
-        
-            
+
+
 class ColorBrush(object):
     def __init__(self, color):
         # fixme: what format is the color passed in? int or float
         self.color = color
-        # fixme: This was needed for a font setting in document.  
+        # fixme: This was needed for a font setting in document.
         #        Fix this and remove.
         self.Colour = self.color
-    
+
     def __repr__(self):
         return 'ColorBrush(%r)' % (self.color,)
 
     def IsOk(self):
-        return True    
+        return True
 
     def set_on_gc(self, gc):
         """ Set the appropriate properties on the GraphicsContext.
@@ -134,20 +133,6 @@ class ColorBrush(object):
         color = tuple([x/255.0 for x in self.color])
         gc.set_fill_color(color)
 
-class AbstractGradientBrush(object):
-    """ Abstract base class for gradient brushes so they can be detected easily.
-    """
-
-    def IsOk(self):
-        return True
-
-    def bbox_transform(self, gc, bbox):
-        """ Apply a transformation to make the bbox a unit square.
-        """
-        x0, y0, w, h = bbox
-        gc.concat_ctm(((w, 0, 0), (0, h, 0), (x0, y0, 1)))
-
-
 class LinearGradientBrush(AbstractGradientBrush):
     """ A Brush representing a linear gradient.
     """
@@ -168,26 +153,42 @@ class LinearGradientBrush(AbstractGradientBrush):
                 self.spreadMethod, self.transforms, self.units))
 
     def set_on_gc(self, gc, bbox=None):
+
+        # Apply transforms
+        if self.transforms is not None:
+            for func, args in self.transforms:
+                func(gc, *args)
+                
+
+        x1 = self.x1
+        x2 = self.x2
+        y1 = self.y1
+        y2 = self.y2
+
         if sys.platform == 'darwin':
-            from enthought.kiva.mac.ABCGI import AxialShading, PiecewiseLinearColorFunction
             if self.spreadMethod != 'pad':
                 warnings.warn("spreadMethod %r is not supported. Using 'pad'" % self.spreadMethod)
-            if self.transforms is not None:
-                for func, args in self.transforms:
-                    func(gc, *args)
-            func = PiecewiseLinearColorFunction(self.stops)
-            if self.units == 'objectBoundingBox' and bbox is not None:
-                self.bbox_transform(gc, bbox)
-            shading = AxialShading(func, (self.x1,self.y1), (self.x2,self.y2),
-                extend_start=1, extend_end=1)
-            gc.draw_shading(shading)
+
+            if bbox is not None:
+                gc.clip_to_rect(*bbox)
+
         else:
-            if not hasattr(gc, 'linear_gradient'):
-                warnings.warn("Gradients for this platform is not implemented.")
-            else:
-                gc.linear_gradient(self.x1, self.y1, self.x2, self.y2, self.stops,
-                                    self.spreadMethod, self.transforms,
-                                    self.units)
+            if self.units == 'objectBoundingBox' and bbox is not None:
+                x1 = (bbox[2] + bbox[0])*x1
+                y1 = (bbox[3] + bbox[1])*y1
+                x2 = (bbox[2] + bbox[0])*x2
+                y2 = (bbox[3] + bbox[1])*y2
+                
+            # kiva and SVGs have different origins, so flip the points
+            y1 = gc.height() - y1
+            y2 = gc.height() - y2
+
+        if self.units == 'objectBoundingBox' and bbox is not None:
+            self.bbox_transform(gc, bbox)
+                
+        stops = np.transpose(self.stops)
+        gc.linear_gradient(x1, y1, x2, y2, stops, self.spreadMethod)            
+
 
 class RadialGradientBrush(AbstractGradientBrush):
     """ A Brush representing a radial gradient.
@@ -215,72 +216,90 @@ class RadialGradientBrush(AbstractGradientBrush):
                 self.transforms, self.units))
 
     def set_on_gc(self, gc, bbox=None):
+        
+        if self.transforms is not None:
+            for func, args in self.transforms:
+                func(gc, *args)
+
+        cx = self.cx
+        cy = self.cy
+        r = self.r
+        fx = self.fx
+        fy = self.fy
+        
         if sys.platform == 'darwin':
-            from enthought.kiva.mac.ABCGI import RadialShading, PiecewiseLinearColorFunction
             if self.spreadMethod != 'pad':
                 warnings.warn("spreadMethod %r is not supported. Using 'pad'" % self.spreadMethod)
-            if self.transforms is not None:
-                for func, args in self.transforms:
-                    func(gc, *args)
-            func = PiecewiseLinearColorFunction(self.stops)
-            if self.units == 'objectBoundingBox' and bbox is not None:
-                self.bbox_transform(gc, bbox)
-            shading = RadialShading(func, (self.fx,self.fy), 0.0,
-                (self.cx,self.cy), self.r, extend_start=1, extend_end=1)
-            gc.draw_shading(shading)
+
+            if bbox is not None:
+                gc.clip_to_rect(*bbox)
+                
         else:
-            if not hasattr(gc, 'radial_gradient'):
-                warnings.warn("Gradients for this platform is not implemented.")
-            else:
-                gc.radial_gradient(self.cx, self.cy, self.r, self.stops, self.fx,
-                                    self.fy, self.spreadMethod, self.transforms,
-                                    self.units)
+            if self.units == 'objectBoundingBox' and bbox is not None:
+                cx = (bbox[2] + bbox[0])*cx
+                cy = (bbox[3] + bbox[1])*cy
+                fx = (bbox[2] + bbox[0])*fx
+                fy = (bbox[3] + bbox[1])*fy
+                r *= np.sqrt((bbox[2] - bbox[0])**2 + (bbox[3] - bbox[1])**2)
+
+            # kiva and SVGs have different origins, so flip the points
+            cy = gc.height()-cy
+            fy = gc.height()-fy
+
+        if self.units == 'objectBoundingBox' and bbox is not None:
+            self.bbox_transform(gc, bbox)
+
+            pass
+
+                            
+        stops = np.transpose(self.stops)
+        gc.radial_gradient(cx, cy, r, fx, fy, stops, self.spreadMethod)
 
 
 def font_style(font):
-    """ Return a string for the font style of a given font.  
-        
+    """ Return a string for the font style of a given font.
+
         fixme: Shouldn't the backends handle this?
     """
-    
+
     if font.style == 'italic' and font.weight == 'bold':
         style = 'bold italic'
     elif font.style == 'italic':
         style = 'italic'
-    elif font.weight== 'bold':                    
+    elif font.weight== 'bold':
         style = 'bold'
     elif font.style in [0, 'regular','normal']:
         style = 'regular'
     else:
         print "Font style '%s' and weight: '%s' not known." \
-              " Using style='regular'" % (font.style, font.weight)           
+              " Using style='regular'" % (font.style, font.weight)
         style = 'regular'
 
     return style
-    
+
 class Renderer(NullRenderer):
     # fimxe: Shouldn't this just be the GraphicsContext?
-    
+
     NullBrush = None
-    
+
     NullGraphicsBrush = None
     NullPen = None
     TransparentPen = Pen((1.0, 1.0, 1.0, 0.0))
-    
+
     caps = {
             'butt':kiva.CAP_BUTT,
             'round':kiva.CAP_ROUND,
             'square':kiva.CAP_SQUARE
             }
-    
+
     joins = {
             'miter':kiva.JOIN_MITER,
             'round':kiva.JOIN_ROUND,
             'bevel':kiva.JOIN_BEVEL
             }
-    
+
     fill_rules = {'nonzero':kiva.FILL, 'evenodd': kiva.EOF_FILL}
-    
+
     def __init__(self):
         pass
 
@@ -297,17 +316,17 @@ class Renderer(NullRenderer):
     @classmethod
     def createBrush(cls, color_tuple):
         return ColorBrush(color_tuple)
-    
+
     @classmethod
     def createNativePen(cls, pen):
         # fixme: Not really sure what to do here...
         #return wx.GraphicsRenderer_GetDefaultRenderer().CreatePen(pen)
         return pen
-        
+
     @classmethod
     def createPen(cls, color_tuple):
         return Pen(color_tuple)
-    
+
     @classmethod
     def createLinearGradientBrush(cls, x1,y1,x2,y2, stops, spreadMethod='pad',
                                   transforms=None, units='userSpaceOnUse'):
@@ -316,11 +335,11 @@ class Renderer(NullRenderer):
 
     @classmethod
     def createRadialGradientBrush(cls, cx,cy, r, stops, fx=None,fy=None,
-                                  spreadMethod='pad', transforms=None, 
+                                  spreadMethod='pad', transforms=None,
                                   units='userSpaceOnUse'):
         return RadialGradientBrush(cx,cy, r, stops, fx,fy, spreadMethod,
             transforms, units)
-        
+
     @classmethod
     def getCurrentPoint(cls, path):
         return path.GetCurrentPoint()
@@ -339,25 +358,26 @@ class Renderer(NullRenderer):
 
     @classmethod
     def makeMatrix(cls, *args):
-        return wx.GraphicsRenderer_GetDefaultRenderer().CreateMatrix(*args)
-    
+        raise NotImplemented()
+
     @classmethod
     def makePath(cls):
-        return CompiledPath()        
+        return CompiledPath()
 
     @classmethod
     def popState(cls, gc):
         return gc.restore_state()
-    
+
     @classmethod
     def pushState(cls, gc):
         return gc.save_state()
-    
+
 
     @classmethod
     def setFontSize(cls, font, size):
-        font.size = size
-        return font        
+        # Agg expects only integer fonts
+        font.size = int(size)
+        return font
 
     @classmethod
     def setFontStyle(cls, font, style):
@@ -368,7 +388,7 @@ class Renderer(NullRenderer):
                 font.style = kiva.fonttools.font.font_styles[style]
         else:
             font.style = style
-    
+
     @classmethod
     def setFontWeight(cls, font, weight):
         if isinstance(weight, basestring):
@@ -385,7 +405,7 @@ class Renderer(NullRenderer):
 
         # text color is controlled by stroke instead of fill color in kiva.
         gc.set_stroke_color(color)
-        
+
         try:
             # fixme: The Mac backend doesn't accept style/width as non-integers
             #        in set_font, but does for select_font...
@@ -394,15 +414,16 @@ class Renderer(NullRenderer):
                 gc.select_font(font.face_name, font.size, style=style)
             else:
                 gc.set_font(font)
-            
+
+
         except ValueError:
             warnings.warn("failed to find set '%s'.  Using Arial" % font.face_name)
             if sys.platform == 'darwin':
                 style = font_style(font)
-                gc.select_font('Arial', font.size, style)    
+                gc.select_font('Arial', font.size, style)
             else:
                 gc.set_font(font)
-            
+
     @classmethod
     def setBrush(cls, gc, brush):
         if brush is Renderer.NullBrush:
@@ -414,11 +435,11 @@ class Renderer(NullRenderer):
     @classmethod
     def setPen(cls, gc, pen):
         pen.set_on_gc(gc)
-        
+
     @classmethod
     def setPenDash(cls, pen, dasharray, offset):
         pen.SetDash(dasharray, offset)
-    
+
     @classmethod
     def strokePath(cls, gc, path):
         # fixme: Do we need to clear the path first?
@@ -435,9 +456,20 @@ class Renderer(NullRenderer):
     def gradientPath(cls, gc, path, brush):
         gc.save_state()
         gc.add_path(path)
-        gc.clip()
-        bbox = path.get_bounding_box()
+        #gc.clip()
+        if hasattr(path, 'get_bounding_box'):
+            bbox = path.get_bounding_box()
+        else:
+            bbox = [np.inf, np.inf, -np.inf, -np.inf]
+            for i in range(path.total_vertices()):
+                vertex = path.vertex(i)
+                bbox[0] = min(bbox[0], vertex[0][0])
+                bbox[1] = min(bbox[1], vertex[0][1])
+                bbox[2] = max(bbox[2], vertex[0][0])
+                bbox[3] = max(bbox[3], vertex[0][1])
+
         brush.set_on_gc(gc, bbox=bbox)
+        gc.fill_path()
         gc.restore_state()
 
     #@classmethod
@@ -458,7 +490,7 @@ class Renderer(NullRenderer):
     @classmethod
     def scale(cls, gc, sx, sy):
         return gc.scale_ctm(sx, sy)
-    
+
     @classmethod
     def GetTextExtent(cls, gc, text):
         x, y, w, h = gc.get_text_extent(text)
@@ -468,7 +500,7 @@ class Renderer(NullRenderer):
     def DrawText(cls, gc, text, x, y, brush, anchor='start'):
         """ Draw text at the given x,y position with the color of the
             given brush.
-            
+
             fixme: Handle gradients...?
         """
         gc.save_state()
@@ -477,21 +509,21 @@ class Renderer(NullRenderer):
             # Setting stroke instead of fill color because that is
             # what kiva uses.
             gc.set_stroke_color(color)
-        
+
             # PDF (our API) has the origin in the lower left.
             # SVG (what we are rendering) has the origin in the upper right.
-            # The ctm (our API) has been set up with a scaling and translation to 
-            # draw as if the upper right is the origin and positive is down so 
-            # that the SVG will render correctly.  This works great accept for 
+            # The ctm (our API) has been set up with a scaling and translation to
+            # draw as if the upper right is the origin and positive is down so
+            # that the SVG will render correctly.  This works great accept for
             # text which will render up side down.  To fix this, we set the
             # text transform matrix to have y going up so the text is rendered
             # upright.  But since, +y is now *up*, we need to draw at -y.
-        
+
             # fixme: There is something wrong with the text matrix.  The following
             #        commands don't work and I would expect them to.
             #text_matrix = affine.affine_from_values(1,0,0,-1,x,-y)
             #gc.set_text_matrix(text_matrix)
-            #gc.show_text_at_point(text, 0, 0)        
+            #gc.show_text_at_point(text, 0, 0)
 
             if anchor != 'start':
                 tx, ty, tw, th = gc.get_text_extent(text)
@@ -501,9 +533,9 @@ class Renderer(NullRenderer):
                     x -= tw
             gc.scale_ctm(1.0, -1.0)
             gc.show_text_at_point(text, x, -y)
-        finally:    
+        finally:
             gc.restore_state()
-    
+
     @classmethod
     def DrawImage(cls, gc, image, x, y, width, height):
         rect = (x, y, width, height)
